package com.gyee.alarm.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gyee.alarm.init.CacheContext;
import com.gyee.alarm.model.auto.*;
import cn.hutool.core.bean.BeanUtil;
import com.gyee.alarm.model.auto.ProBasicFeatureStat;
import com.gyee.alarm.model.vo.AlarmType;
import com.gyee.alarm.service.auto.IAlarmTsService;
import com.gyee.alarm.service.auto.IProBasicFeatureStatService;
import com.gyee.alarm.util.DateUtils;
import com.gyee.alarm.util.SnowflakeGenerator;
import com.gyee.alarm.util.StringUtils;


import javafx.scene.chart.CategoryAxis;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.*;
import java.util.stream.Collectors;

@Service

public class FeatureStatService {
    @Resource
    private IAlarmTsService alarmTsService;

    @Resource
    private IProBasicFeatureStatService proBasicFeatureStatService;

    public void findFeatureStatList(Date recordDate) {


        Calendar c = Calendar.getInstance();
        c.setTime(recordDate);
        Date begin = DateUtils.truncate(c.getTime());
        Date end = DateUtils.addDays(begin, 1);

        Date statDate=DateUtils.truncate(recordDate);

        QueryWrapper<ProBasicFeatureStat> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("stat_date", statDate);
        //判断是否有重复记录，先删除重复记录
        List<String> idls = proBasicFeatureStatService.list(queryWrapper).stream()
                .map(ProBasicFeatureStat::getId)
                .collect(Collectors.toList());

        if (idls.size() > 0) {

            List<String> templs = new ArrayList<>();
            for (String vo : idls) {
                templs.add(vo);
                if (templs.size() == 100) {
                    proBasicFeatureStatService.removeByIds(templs);
                    templs = new ArrayList<>();
                }

            }

            if (!templs.isEmpty()) {
                proBasicFeatureStatService.removeByIds(templs);
            }

        }

        List<ProBasicFeatureStat> fsls = alarmTsService.findFeatureStatList(begin.getTime(), end.getTime());

        Map<String, ProBasicFeatureStat> map = new HashMap<>();


        if (!fsls.isEmpty()) {
            for (ProBasicFeatureStat fs : fsls) {
                if (StringUtils.notEmp(fs.getTypeCode()) && CacheContext.alarmTypeMap.containsKey(fs.getTypeCode())) {
                    ProEconAlarmType at = CacheContext.alarmTypeMap.get(fs.getTypeCode());
                    fs.setTypeName(at.getName());
                }
                fs.setStatDate(new Timestamp(statDate.getTime()));
                fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
                StringBuilder sb = new StringBuilder();
                sb.append(fs.getWtId()).append("_").append(fs.getAlarmid());
                map.put(String.valueOf(sb), fs);
            }
        }
        List<ProBasicFeatureStat> vos = new ArrayList<>();
        for (ProBasicEquipment wt : CacheContext.wtls) {

            StringBuilder sb=new StringBuilder();
            sb.append(wt.getWindpowerstationId()).append(wt.getModelId());
            if(CacheContext.wtAlarmMap.containsKey(String.valueOf(sb))) {
                List<ProEconAlarmConfiguration> ls = CacheContext.wtAlarmMap.get(String.valueOf(sb));
                for (ProEconAlarmConfiguration at : ls) {

                    setValues(statDate, map, vos, wt, at);
                }
            }

        }

        List<ProBasicFeatureStat> templs = new ArrayList<>();
        for (ProBasicFeatureStat vo : vos) {
            templs.add(vo);
            if (templs.size() == 100) {
                proBasicFeatureStatService.saveBatch(templs);
                templs = new ArrayList<>();
            }

        }

        if (!templs.isEmpty()) {
            proBasicFeatureStatService.saveBatch(templs);
        }


    }

    private  static void setValues(Date statDate,Map<String, ProBasicFeatureStat> map, List<ProBasicFeatureStat> vos, ProBasicEquipment wt, ProEconAlarmConfiguration at) {

        ProBasicFeatureStat fs = new ProBasicFeatureStat();

        StringBuilder sb = new StringBuilder();
        sb.append(fs.getWtId()).append("_").append(at.getId());
        if (map.containsKey(String.valueOf(sb))) {
            ProBasicFeatureStat old = map.get(String.valueOf(sb));
            BeanUtil.copyProperties(old, fs);
        } else {
            fs.setStatDate(new Timestamp(statDate.getTime()));
            fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
            fs.setWtId(wt.getId());
            fs.setStationId(wt.getWindpowerstationId());
            fs.setTypeCode(at.getComponents());
            if(CacheContext.alarmTypeMap.containsKey(at.getComponents()))
            {
                fs.setTypeName(CacheContext.alarmTypeMap.get(at.getComponents()).getName());
            }
            fs.setAlarmid(at.getId());
            fs.setDescription(at.getDescription());
            fs.setTotal(0l);
            fs.setAvgSeconds(0.0);
            fs.setMaxSeconds(0.0);
            fs.setMinSeconds(0.0);
            fs.setTotalSeconds(0.0);

        }
        vos.add(fs);
    }


}
