package com.gyee.alarm.controller;

import com.gyee.alarm.model.vo.AjaxResult;
import com.gyee.alarm.model.vo.AjaxStatus;
import com.gyee.alarm.model.vo.AlarmStatVo;
import com.gyee.alarm.model.vo.AlarmVo;
import com.gyee.alarm.service.AlarmHistoryService;

import com.gyee.alarm.util.DateUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


@Slf4j
@RestController
@RequestMapping("/alarm")
@CrossOrigin
public class AlarmHistoryController {


    Logger logger = LogManager.getLogger(AlarmHistoryController.class);

    @Resource
    private AlarmHistoryService alarmHistoryService;



    @GetMapping(value = "/history/findAlarmlist")
    @ApiOperation(value = "报警信息查询", notes = "报警信息查询")
    @ApiImplicitParams({

            @ApiImplicitParam(name = "begin", value = "开始时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "end", value = "结束时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "alarmType", value = "报警类型 booststation:升压站报警,windturbine:设备报警,custom:自定义报警", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "stationid", value = "场站编号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "deviceid", value = "设备编号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "description", value = "描述", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "modelId", value = "型号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "deviceType", value = "报警类型 booststation:升压站报警,windturbine:设备报警", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "components", value = "部件编号", required = false, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "pageNum", value = "页号", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页显示行数", required = true, dataType = "string", paramType = "query")})
    public AjaxResult findAlarmlist(
            @RequestParam(value = "begin", required = true) String begin,
            @RequestParam(value = "end", required = true) String end,
            @RequestParam(value = "alarmType", required = true) String alarmType,
            @RequestParam(value = "stationid", required = false) String stationid,
            @RequestParam(value = "deviceid", required = false) String deviceid,
            @RequestParam(value = "description", required = false) String description,
            @RequestParam(value = "modelId", required = false) String modelId,
            @RequestParam(value = "deviceType", required = false) String deviceType,
            @RequestParam(value = "components", required = false) String components,
            @RequestParam(value = "pageNum", required = true) Integer pageNum,
            @RequestParam(value = "pageSize", required = true) Integer pageSize

    ) throws Exception {

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Date beginDate = df.parse(begin);
        Date endDate = df.parse(end);
        Map<String,Object> map = alarmHistoryService.findAlarmlist(modelId,beginDate.getTime(), endDate.getTime(), alarmType, stationid,deviceid, description,deviceType,components, pageNum, pageSize);
        return AjaxResult.successData(AjaxStatus.success.code, map);

    }

    @GetMapping(value = "/history/findAlarmByTypelist")
    @ApiOperation(value = "获取升压站、设备、自定义报警报警信息查询", notes = "报警信息查询")
   public AjaxResult findAlarmByTypelist() throws Exception {


        Map<String,Object> map = alarmHistoryService.findAlarmByTypelist();
        return AjaxResult.successData(AjaxStatus.success.code, map);

    }


    @GetMapping(value = "/history/findFeatureStat")
    @ApiOperation(value = "查询报警类型、报警时长、报警次数", notes = "查询报警类型、报警时长、报警次数")
    @ApiImplicitParams({

            @ApiImplicitParam(name = "begin", value = "开始时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "end", value = "结束时间", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "stationid", value = "场站编号", required = true, dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "components", value = "部件编号", required = true, dataType = "string", paramType = "query")
           })
    public List<Map.Entry<String, List<AlarmStatVo>>> findFeatureStat(

            @RequestParam(value = "begin", required = true) String begin,
            @RequestParam(value = "end", required = true) String end,
            @RequestParam(value = "stationid", required = true) String stId,
            @RequestParam(value = "components", required = true) String components
    ) throws ParseException {


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date dtStart = sdf.parse(begin);
        Date dtEnd = sdf.parse(end);
        List<AlarmStatVo> pressureList = alarmHistoryService.findByCondition(stId, dtStart, dtEnd,components);

//
//        Map<String, List<AlarmStatVo>> map=new TreeMap<>();
//
//        for(AlarmStatVo vo:pressureList)
//        {
//            if(map.containsKey(vo.getRelateParts()))
//            {
//                List<AlarmStatVo>
//            }
//        }
        //根据风机编号分组
        Map<String, List<AlarmStatVo>> alarmStatMapGroupbyWindturbineId = pressureList.stream().collect(Collectors.groupingBy(AlarmStatVo::getWindturbineId));

        Map<String, List<AlarmStatVo>> resultMap = new TreeMap<>();

        for (Map.Entry<String, List<AlarmStatVo>> entry : alarmStatMapGroupbyWindturbineId.entrySet()) {
            String windturbineId = entry.getKey();
            resultMap.put(windturbineId, entry.getValue());
        }

        //根据风机编号进行排序
        List<Map.Entry<String, List<AlarmStatVo>>> list = new ArrayList<>(resultMap.entrySet());
        Collections.sort(list, new Comparator<Map.Entry<String, List<AlarmStatVo>>>() {
            @Override
            public int compare(Map.Entry<String, List<AlarmStatVo>> o1, Map.Entry<String, List<AlarmStatVo>> o2) {
               String key1=o1.getKey();
               String key2=o1.getKey();
               return key1.compareTo(key2);
            }
        });

        return list;


    }

}




